<?php
namespace Woopromotion\Classes;
 /**
  * 
  * @package    WooPromotion
  * @version    1.0
  * @author     WPBucket
  * @Websites: http://wpbucket.net
  *
  */
 
  if( ! defined( 'ABSPATH' ) ) {
      die( WOOPROMOTION_ALERT_MSG );
  }

class Badge_Manager extends Product_Data {

  public static function getBadges() {
    self::badgeHtml();
  }
  private static function getBadgeIds() {
    $b = new \Woopromotion\Classes\Badges();
    return $b->getBadgeIds();
  }

  public static function badgeHtml() {

    $badges = self::getBadgeMaping();
    //
    $hoverClass = \Woopromotion\Inc\Helper::getOption('badge_hover_scaleup') ? ' hover-scale' :'';
    //
    if( !empty( $badges ) ) {
      $class = is_product() ? 'product-summary-baddge' : 'shop-product-baddge';
      echo '<div class="woopromotion-product-baddge '.esc_attr( $class.$hoverClass ).'">';
      foreach( $badges as $badge ){
        echo '<div class="woopromotion-product-baddge-inner"><img src="'.esc_url( $badge['url'] ).'" data-tooltip="'.esc_attr( $badge['name'] ).'" /></div>';
      }
      echo '</div>';
    }

  }

  private static function getBadgeMaping() {

    $ids = self::getBadgeIds();

    $badges = [];

      if( !empty( self::getProductBadgeId() ) && is_array( self::getProductBadgeId() ) ) {
        foreach( self::getProductBadgeId() as $id ) {
          $badges[] = self::getBadgeOnProduct( $id );
        }
      } 

      //
      if( !empty( $ids ) ) {
        foreach( $ids as $id ) {
          $type = get_post_meta( $id, 'woopromotion_badge_type', true );

          switch( $type ) {
            case 'category':
              $badges[] = self::getBadgeOnCategory($id);
            break;
            case 'rating':
              $badges[] = self::getBadgeOnRating($id);
            break;
            case 'featured':
              $badges[] = self::getBadgeOnFeatured($id);
            break;
            case 'sales':
              $badges[] = self::getBadgeOnSale( $id );
            break;
            case 'stock':
              $badges[] = self::getBadgeOnStock($id);
            break;
            case 'sales_rating':
              $badges[] = self::getBadgeOnRatingSales($id);
            break;
          }
        }
      }

    return array_filter($badges);
  }
  private static function getProductBadgeId() {
    return get_post_meta( get_the_ID(), '_woopromotion_product_badge', true );
  }
  private function getCurrentStock() {
    $product = $this->getProduct;
    $is_manage_stock = $product->get_manage_stock();
    if( !empty( $is_manage_stock ) ) {
      return $product->get_stock_quantity();
    } else {
      return 'no';
    }
   
  }

  private function getTotalSales() {
    $product = $this->getProduct;
    return $product->get_total_sales();
  }
  private function getProductTotalRating() {
    $product = $this->getProduct;
    return $product->get_rating_count();  
  }
  private function getProductAverageRating() {
    $product = $this->getProduct;
    return $product->get_average_rating();  
  }
  private function getFeaturedProduct() {
    $product = $this->getProduct;
    return $product->get_featured(); 
  }
  private static function getCategory() {
    return get_the_terms( get_the_ID(), 'product_cat' );
  }

  private static function getBadgeOnProduct( $id ) {
    return self::getBadgeInfoById( $id );
  }
  private static function getBadgeOnStock( $badgeId ) {
    $productStock = (new self())->getCurrentStock();
    
    if( $productStock !== 'no' ) {
      $badgeStockLimit = get_post_meta( $badgeId, 'woopromotion_less_stock', true);
      if( $productStock < $badgeStockLimit ) {
        return self::getBadgeInfoById( $badgeId );
      }

    }

  }
  private static function getBadgeOnCategory( $badgeId ) {

    $terms = (new self())->getCategory();
    if( !is_array( $terms ) && empty( $terms ) ) {
      return;
    }

    $cats = array_column( $terms, 'slug' );
    $cat = get_post_meta( $badgeId, 'woopromotion_product_category', true);

    if( in_array( $cat, $cats ) ) {
      return self::getBadgeInfoById( $badgeId );
    }

  }

  private static function getBadgeOnFeatured( $badgeId ) {

    $is_featured = (new self())->getFeaturedProduct();
    if( empty( $is_featured ) ) {
      return;
    }
    return self::getBadgeInfoById( $badgeId );
  }

  private static function getBadgeOnSale( $badgeId ) {

    $totalSale = (new self())->getTotalSales();
    $saleLimit = get_post_meta( $badgeId, 'woopromotion_sold_min_limit', true);
    if( $totalSale >= $saleLimit  ) {
      return self::getBadgeInfoById( $badgeId );
    }
    
  }

  private static function getBadgeOnRatingSales( $badgeId ) {

    $totalSale      = (new self())->getTotalSales();
    $totalReviews   = (new self())->getProductTotalRating();
    $averageRating  = (new self())->getProductAverageRating();


    $minimumReviews = get_post_meta( $badgeId, 'woopromotion_minimum_reviews', true);
    $minRatings = get_post_meta( $badgeId, 'woopromotion_product_min_ratings', true);
    $soldLimit = get_post_meta( $badgeId, 'woopromotion_sold_min_limit', true);

    if( ( $totalSale >= $soldLimit ) && ( $totalReviews >= $minimumReviews ) && ( $averageRating >= $minRatings )  ) {
      return self::getBadgeInfoById( $badgeId );
    }
  }

  private static function getBadgeOnRating( $badgeId ) {

    $totalReviews = (new self())->getProductTotalRating();
    $averageRating = (new self())->getProductAverageRating();

    $minimumReviews = get_post_meta( $badgeId, 'woopromotion_minimum_reviews', true);
    $minRatings = get_post_meta( $badgeId, 'woopromotion_product_min_ratings', true);

    if( $totalReviews >= $minimumReviews  && $averageRating >= $minRatings  ) {
      return self::getBadgeInfoById( $badgeId );
    }
    
  }

  public static function getBadgeInfoById($badgeId) {
    if( !empty( $badgeId ) ) {
      return [ 'name' => get_the_title( $badgeId ), 'url' => get_the_post_thumbnail_url( $badgeId ) ];
    }
  }

}